"""
.. _ad_from_string:

Create Ads Using Long Descriptive Text (top-down approach)
==========================================================

Many times you have long descriptive text about your products, especially on
their respective landing pages. The allowed length of text ads has become
considerably long on many platforms. On Google Ads for example, you have slots
of 30, 30, 30, 90, and 90 characters, for a total of 270. That's more than
enough space to talk about the main features of your product.

The question is, how do you utilize that long description text that has all the
details that you want, and make sure it fits correctly within the limits given
by the platform you are using?

The :func:`ad_from_string` function does exactly that. Given a long string, it
divides it into slots of any given length that you specify, and if any text
remains it will be appended to the end of the returned list.

Another important benefit of this is that you can take those long descriptions
(or write them) once, and then you can easily split them into different slots
based on the ad format and the platform you are using.

Here is a quick overview of the available parameters and options:

.. glossary::

    s
        The string that you want to split. This would typically be available
        on the landing pages of each product.

    slots

        The lengths that you want to split into. Note that although the default
        uses Google Ads' text ad template, you can change it to any other
        group of slots, with more or fewer slots of different lengths.

    sep

        The separator by which to split the text. The default is :attr:`None`
        which splits the text by whitespace, but you can change it to something
        else if needed. Sometimes you might want the text split by hyphens
        (URLs for example) so you can split by that character.

    capitalize

        The default is :attr:`False` which leaves the capitalization of
        :attr:`s` intact. If you set it to :attr:`True` then the first letter
        of each word would be capitalized.


**Example**

Note that in any case, the returned list of characters is longer than the
provided slots by one. So if you provide five slots, for example, the function
will always return a list of length six.

This is to ensure that the remainder of the text is not lost if it is longer,
so you know what is missing. In case you have shorter text, you will still have
one element more than the provided slots to ensure consistency.

>>> desc_text = "Get the latest gadget online. The GX12 model comes with 13 things that do a lot of good stuff for your health. Start shopping now."
>>> len(desc_text)
130

Now let's see how this same description can be utilized in different scenarios

Google Text Ads
^^^^^^^^^^^^^^^

Since this is shorter than the default Google values, you will get extra empty
slots (with an additional last one).

>>> ad_from_string(desc_text)  # default values (Google text ads)
['Get the latest gadget online.',
 'The GX12 model comes with 13',
 'things that do a lot of good',
 'stuff for your health. Start shopping now.',
 '',
 '',
 '',
 '']

Facebook Feed Ads
^^^^^^^^^^^^^^^^^

In this case, it is also shorter than the default value, so you get an extra
space.

>>> ad_from_string(desc_text, [125, 25, 30])  # Facebook feed ads
['Get the latest gadget online. The GX12 model comes with 13 things that do a lot of good stuff for your health. Start shopping',
 'now.',
 '',
 '']

Since it might not look good having just one word in the second slot, and an
empty last one, you might want to change it as follows:

>>> ad_from_string(desc_text, [90, 25, 30])
['Get the latest gadget online. The GX12 model comes with 13 things that do a lot of good',
 'stuff for your health.',
 'Start shopping now.',
 '']


Facebook Instant Article Ad
^^^^^^^^^^^^^^^^^^^^^^^^^^^
Here is a case where our text is longer than the provided limitations, so we
end up having an extra space that is not used:

>>> ad_from_string(desc_text, [25, 30])  # Facebook instant article ad
['Get the latest gadget',
 'online. The GX12 model comes',
 'with 13 things that do a lot of good stuff for your health. Start shopping now.']
"""
import string


def ad_from_string(s, slots=(30, 30, 30, 90, 90, 15, 15), sep=None,
                   capitalize=False):
    """Convert string :attr:`s` to an ad by splitting it into groups of words.
    Each group would have a length of at most the allowed length for that slot.

    If the total length of :attr:`s` exceeds the total allowed length, all
    remaining characters would be grouped in the last element of the
    returned list.

    :param str s: a string of characters, with no restrictions on length
    :param list slots: an iterable of integers for the maximum lengths for
        each slot
    :param str sep: character(s) by which to split :attr:`s`
    :param bool capitalize: whether or not to capitalize each word after
                            grouping. Setting it as False would not change the
                            capitalization of the input string
    :returns text_ad: a list of strings

    >>> ad_from_string('this is a short ad')
    ['this is a short ad', '', '', '', '', '', '', '']

    >>> ad_from_string('this is a longer ad and will take the first two slots')
    ['this as a longer ad and would', 'take the first two slots',
    '', '', '', '', '', '']

    >>> ad_from_string("Slots can be changed the way you want", (10, 15, 10))
    ['Slots can', 'be changed the', 'way you', 'want']

    >>> ad_from_string("The capitalization REMAinS as IS bY DefAULt",
    ...                       (10, 15, 10))
    ['The', 'capitalization', 'REMAinS as', 'IS bY DefAULt']

    >>> ad_from_string("set captialize=True to capitalize first letters",
    ...                capitalize=True)
    ['Set Captialize=true To', 'Capitalize First Letters',
     '', '', '', '', '', '']
    """
    str_words = s.split(sep=sep)
    text_ad = ['' for x in range(len(slots)+1)]
    counter = 0

    for i, slot in enumerate(slots):
        while counter <= len(str_words) - 1:
            if len(text_ad[i] + str_words[counter]) + 1 > slot:
                break
            text_ad[i] += (' ' + str_words[counter] if text_ad[i]
                           else str_words[counter])
            counter += 1

    text_ad[-1] = (sep.join(str_words[counter:])
                   if sep is not None else ' '.join(str_words[counter:]))

    return [string.capwords(x) if capitalize else x for x in text_ad]



from advertools.sitemaps import sitemap_to_df
import pandas as pd
import pytest

tests_dir = 'https://raw.githubusercontent.com/eliasdabbas/advertools/master/tests/'

sitemap_url = tests_dir + 'sitemap.xml'
sitemap_gz_url = tests_dir + 'sitemap.xml.gz'
sitemap_index_url = tests_dir + 'sitemap-index.xml'


def test_regular_sitemap():
    result = sitemap_to_df(sitemap_url)
    assert isinstance(result, pd.core.frame.DataFrame)
    assert len(result) == 5


# def test_gz_sitemap():
#     result = sitemap_to_df(sitemap_gz_url)
#     assert isinstance(result, pd.core.frame.DataFrame)
#     assert len(result) == 5


def test_sitemap_index():
    result = sitemap_to_df(sitemap_index_url)
    assert isinstance(result, pd.core.frame.DataFrame)
    assert len(result) == 6



from advertools.word_frequency import word_frequency


text_list = [
    'one two',
    'one two  three',
    'one-two-three',
    'one_two_three',
    'four five',
    'four five',
    'four six'
]

num_list = [
    100,
    200,
    300,
    400,
    500,
    600,
    700
]


def test_rm_words_removed():
    result = word_frequency(text_list, num_list, rm_words=['one', 'two'])
    assert not result['word'].eq('one').any()
    assert not result['word'].eq('two').any()


def test_extra_info_not_provided():
    result = word_frequency(text_list, num_list, extra_info=False)
    assert set(result.columns.values) == {'word', 'abs_freq', 'wtd_freq',
                                          'rel_value'}


def test_extra_info_provided():
    result = word_frequency(text_list, num_list, extra_info=True)
    assert set(result.columns.values) == {'word', 'abs_freq', 'abs_perc',
                                          'abs_perc_cum', 'wtd_freq',
                                          'wtd_freq_perc', 'wtd_freq_perc_cum',
                                          'rel_value'}


def test_works_fine_with_only_stopwords_supplied():
    result = word_frequency(['on'], [3])
    assert result.shape == (0, 4)


def test_works_without_numlist_provided():
    result = word_frequency(['Great Text in a List', 'Greater text as well'])
    assert result['word'].eq('text').any()


def test_word_freq_uses_regex():
    result = word_frequency(['pizza burger', 'pizza sandwitch'], regex='pizza')
    assert result['word'][0] == 'pizza'



import pytest

from advertools.extract import * # (extract, extract_currency, extract_emoji,
#                                 extract_exclamations, extract_hashtags,
#                                 extract_intense_words, extract_mentions,
#                                 extract_questions, extract_words, extract_urls)

number_posts = ['before123,000', '123after', 'comma 123,456', 'dot 123.234.3',
                'skip 123-', 'nothing', 'two 123 456,789']

number_summary = extract_numbers(number_posts)

number_test_keys = ['numbers', 'numbers_flat', 'number_counts',
                     'number_freq', 'top_numbers', 'overview']

mention_posts = ['hello @name', 'email@domain.com', '@oneword',
                 'hi @nam-e and @name', '@first @last', 'an @under_score',
                 'a @dot.one', 'non latin @مرحبا', 'other at ＠sign',
                 '@one.@two three', 'number @123text', '@_before @after_']

mention_summary = extract_mentions(mention_posts)

mention_test_keys = ['mentions', 'mentions_flat', 'mention_counts',
                     'mention_freq', 'top_mentions', 'overview']

hashtag_posts = ['hello #name', 'email#domain.com', '#oneword',
                 'hi #nam-e and #name', '#first #last', 'an #under_score',
                 'a #dot.one', 'non latin #مرحبا', 'other hash ＃sign',
                 '#one.#two three', 'number #123text', '#_before #after_']

hashtag_summary = extract_hashtags(hashtag_posts)

hashtag_test_keys = ['hashtags', 'hashtags_flat', 'hashtag_counts',
                     'hashtag_freq', 'top_hashtags', 'overview']

emoji_posts = ['one smiley 😀', 'one smiley 😀 one wink 😉', 'no emoji']

emoji_summary = extract_emoji(emoji_posts)

emoji_test_keys = ['emoji', 'emoji_text', 'emoji_flat', 'emoji_flat_text',
                   'emoji_counts', 'emoji_freq', 'top_emoji', 'top_emoji_text',
                   'top_emoji_groups', 'top_emoji_sub_groups', 'overview']

word_posts = ['today it is raining', 'i like rain and raining',
              'today it is snowing', 'now snowing and raining',
              'training is good with snow', 'RaIn and SNoW',
              'none of the words', '@rain, #snow rain']

word_summary_full = extract_words(word_posts, ['rain', 'snow'], True)

word_summary_not_full = extract_words(word_posts, ['rain', 'snow'], False)

word_test_keys = ['words', 'words_flat', 'word_counts',
                  'word_freq', 'top_words', 'overview']

currency_posts = ['$5.0 beginning', 'mid £5.0 price', 'end of str €',
                  'more $ than £ one', 'other symbols ₣ ₤ ₥ ₦ ₧',
                  'no symbols']

currency_summary = extract_currency(currency_posts)

currency_test_keys = ['currency_symbols', 'currency_symbols_flat',
                      'currency_symbol_counts', 'currency_symbol_freq',
                      'top_currency_symbols', 'overview',
                      'currency_symbol_names', 'surrounding_text']

intense_word_posts = ['i looooove this', 'goooood mooorning',
                      'normal text', 'in thhhhhhe middle',
                      'innnnnn the beginning', 'at the end!!!!!!']

intense_word_summary = extract_intense_words(intense_word_posts)

intense_word_test_keys = ['intense_words', 'intense_words_flat',
                          'intense_word_counts', 'intense_word_freq',
                          'top_intense_words', 'overview']

question_posts = ['how are you?', 'no question', 'no! what about you?',
                  'Hola, ¿cómo estás?', 'Πώς είσαι;']

question_summary = extract_questions(question_posts)

question_test_keys = ['question_marks', 'question_marks_flat',
                      'question_mark_counts', 'question_mark_freq',
                      'top_question_marks', 'overview',
                      'question_mark_names', 'question_text']

exclamation_posts = ['how dare you!', 'no exclamation', 'no! do not do this!',
                     '¡Hola!  ¿cómo estás?', 'مرحبا. لا تذهب!']

exclamation_summary = extract_exclamations(exclamation_posts)

exclamation_test_keys = ['exclamation_marks', 'exclamation_marks_flat',
                         'exclamation_mark_counts', 'exclamation_mark_freq',
                         'top_exclamation_marks', 'overview',
                         'exclamation_mark_names', 'exclamation_text']

url_posts = ['one https://www.a.com', 'two www.a.com www.b.com',
             'nothing', 'long https://example.com/one?a=b#nothing']

url_summary = extract_urls(url_posts)

url_test_keys = ['urls', 'urls_flat', 'url_counts', 'url_freq',
                 'top_urls', 'overview', 'top_domains', 'top_tlds']

test_ids = [
    'currency',
    'emoji',
    'exclamation',
    'hashtag',
    'intense',
    'mention',
    'number',
    'question',
    'word_full',
    'word_not_full',
    'url',
]


summaries_keys = [
    (list(currency_summary.keys()), currency_test_keys),
    (list(emoji_summary.keys()), emoji_test_keys),
    (list(exclamation_summary.keys()), exclamation_test_keys),
    (list(hashtag_summary.keys()), hashtag_test_keys),
    (list(intense_word_summary.keys()), intense_word_test_keys),
    (list(mention_summary.keys()), mention_test_keys),
    (list(number_summary.keys()), number_test_keys),
    (list(question_summary.keys()), question_test_keys),
    (list(word_summary_full.keys()), word_test_keys),
    (list(word_summary_not_full.keys()), word_test_keys),
    (list(url_summary.keys()), url_test_keys),
]


@pytest.mark.parametrize('summary_key, test_key',
                         zip([x[0] for x in summaries_keys],
                             [x[1] for x in summaries_keys]),
                         ids=test_ids)
def test_has_correct_keys(summary_key, test_key):
    assert set(summary_key) == set(test_key)


summaries = {
    'currency': currency_summary,
    'emoji': emoji_summary,
    'exclamation': exclamation_summary,
    'hashtag': hashtag_summary,
    'intense': intense_word_summary,
    'mention': mention_summary,
    'number': number_summary,
    'question': question_summary,
    'word_full': word_summary_full,
    'word_not_full': word_summary_not_full,
    'url': url_summary,
}


summary_counts = {k: [d[key] for key in d if 'count' in key][0]
                  for k, d in summaries.items()}
test_counts = {
    'currency': [1, 1, 1, 2, 5, 0],
    'emoji': [1, 2, 0],
    'exclamation': [1, 0, 2, 2, 1],
    'hashtag': [1, 0, 1, 2, 2, 1, 1, 1, 1, 2, 1, 2],
    'intense': [1, 2, 0, 1, 1, 1],
    'mention': [1, 0, 1, 2, 2, 1, 1, 0, 1, 2, 1, 2],
    'number': [1, 1, 1, 1, 10, 2],
    'question': [1, 0, 1, 2, 1],
    'word_full': [0, 1, 0, 0, 1, 2, 0, 3],
    'word_not_full': [1, 2, 1, 2, 2, 2, 0, 3],
    'url': [1, 2, 0, 1]
}


@pytest.mark.parametrize('summary_count, test_count',
                         zip(sorted(summary_counts.items()),
                             sorted(test_counts.items())),
                         ids=test_ids)
def test_has_correct_counts(summary_count, test_count):
    assert summary_count == summary_count


summary_freqs = {k: [d[key] for key in d if 'freq' in key][0]
                 for k, d in summaries.items()}

test_freqs = {
    'currency': [(0, 1), (1, 3), (2, 1), (5, 1)],
    'emoji': [(0, 1), (1, 1), (2, 1)],
    'exclamation': [(0, 1), (1, 2), (2, 2)],
    'hashtag': [(0, 1), (1, 7), (2, 4)],
    'intense': [(0, 1), (1, 4), (2, 1)],
    'mention': [(0, 2), (1, 6), (2, 4)],
    'number': [(0, 1), (1, 5), (2, 1)],
    'question': [(0, 1), (1, 3), (2, 1)],
    'word_full': [(0, 4), (1, 2), (2, 1), (3, 1)],
    'word_not_full': [(0, 1), (1, 2), (2, 4), (3, 1)],
    'url': [(0, 1), (1, 2), (2, 1)],
}


@pytest.mark.parametrize('summary_freq, test_freq',
                         zip(sorted(summary_freqs.items()),
                             sorted(test_freqs.items())),
                         ids=test_ids)
def test_has_correct_freq(summary_freq, test_freq):
    assert summary_freq == summary_freq


summary_flats = {k: [d[key] for key in d if 'flat' in key
                     and 'emoji_flat_text' not in key][0]
                 for k, d in summaries.items()}
test_flats = {
    'currency': ['$', '£', '€', '$', '£', '₣', '₤', '₥', '₦', '₧'],
    'emoji': ['😀', '😀', '😉'],
    'exclamation': ['!', '!', '!', '¡', '!', '!'],
    'hashtag': ['#name', '#oneword', '#nam', '#name', '#first', '#last',
                '#under_score', '#dot', '#مرحبا', '＃sign', '#one',
                '#two', '#123text', '#_before', '#after_'],
    'intense': ['looooove', 'goooood', 'mooorning', 'thhhhhhe',
                'innnnnn', 'end!!!!!!'],
    'mention': ['@name', '@oneword', '@nam', '@name', '@first',
                '@last', '@under_score', '@dot', '＠sign', '@one',
                '@two', '@123text', '@_before', '@after_'],
    'number': ['123,000', '123', '123,456', '123.234.3', '123',
               '123', '456,789'],
    'question': ['?', '?', '¿', '?', ';'],
    'word_full': ['rain', 'snow', 'rain', 'snow', 'rain', 'snow', 'rain'],
    'word_not_full': ['raining', 'rain', 'raining', 'snowing',
                      'snowing', 'raining', 'training', 'snow',
                      'rain', 'snow', '@rain,', '#snow', 'rain'],
    'url': ['https://www.a.com', 'http://www.a.com',
            'http://www.b.com', 'https://example.com/one?a=b#nothing']
}


@pytest.mark.parametrize('summary_flat, test_flat',
                         zip(sorted(summary_flats.items()),
                             sorted(test_flats.items())),
                         ids=test_ids)
def test_has_correct_flat(summary_flat, test_flat):
    assert summary_flat == test_flat


summary_tops = {k: [d[key] for key in d if 'top' in key]
                for k, d in summaries.items()}

summary_tops = {k: d[0] if len(d) == 1 else d
                for k, d in summary_tops.items()}

test_tops = {
    'currency': [('$', 2), ('£', 2), ('€', 1), ('₣', 1), ('₤', 1),
                 ('₥', 1), ('₦', 1), ('₧', 1)],
    'emoji': [[('😀', 2), ('😉', 1)],
              [('grinning face', 2), ('winking face', 1)],
              [('Smileys & Emotion', 3)],
              [('face-smiling', 3)]],
    'exclamation': [('!', 5), ('¡', 1)],
    'hashtag': [('#name', 2), ('#oneword', 1), ('#nam', 1),
                ('#first', 1), ('#last', 1), ('#under_score', 1),
                ('#dot', 1), ('#مرحبا', 1), ('＃sign', 1), ('#one', 1),
                ('#two', 1), ('#123text', 1), ('#_before', 1),
                ('#after_', 1)],
    'intense': [('looooove', 1), ('goooood', 1), ('mooorning', 1),
                ('thhhhhhe', 1), ('innnnnn', 1), ('end!!!!!!', 1)],
    'mention': [('@name', 2), ('@oneword', 1), ('@nam', 1), ('@first', 1),
                ('@last', 1), ('@under_score', 1), ('@dot', 1),
                ('＠sign', 1), ('@one', 1), ('@two', 1), ('@123text', 1),
                ('@_before', 1), ('@after_', 1)],
    'number': [('123', 3), ('123,000', 1), ('123,456', 1), ('123.234.3', 1),
               ('456,789', 1)],
    'question': [('?', 3), ('¿', 1), (';', 1)],
    'word_full': [('rain', 4), ('snow', 3)],
    'word_not_full': [('raining', 3), ('rain', 3), ('snowing', 2),
                      ('snow', 2), ('training', 1), ('@rain,', 1),
                      ('#snow', 1)],
    'url': [[('https://www.a.com', 1), ('http://www.a.com', 1),
             ('http://www.b.com', 1),
             ('https://example.com/one?a=b#nothing', 1)],
            [('www.a.com', 2), ('www.b.com', 1), ('example.com', 1)],
            [('com', 4)]]
}


def make_hashable(iterable):
    """Some sub-elements are lists and need to be converted sorted tuples."""
    return tuple(tuple(sorted(x)) if isinstance(x, list)else
                 x for x in iterable)


@pytest.mark.parametrize('summary_top, test_top',
                         zip(sorted(summary_tops.items()),
                             sorted(test_tops.items())),
                         ids=test_ids)
def test_has_correct_top(summary_top, test_top):
    assert (set(make_hashable(summary_top[1])) ==
            set(make_hashable(test_top[1])))


summary_overviews = {k: [d[key] for key in d if 'overview' in key][0]
                     for k, d in summaries.items()}

test_overviews = {
    'currency': [6, 10, 10/6, 8],
    'emoji': [3, 3, 1.0, 2],
    'exclamation': [5, 6, 1.2, 2],
    'hashtag': [12, 15, 1.25, 14],
    'intense': [6, 6, 1.0, 6],
    'mention': [12, 14, 14/12, 13],
    'number': [7, 7, 7/7, 5],
    'question': [5, 5, 1.0, 3],
    'word_full': [8, 7, 7/8, 2],
    'word_not_full': [8, 13, 13/8, 7],
    'url': [4, 4, 1.0, 4],
}


def dict2overview_list(d):
    """Convert an overview dict to a list based on its keys."""
    result = [0, 0, 0, 0]
    for key in d:
        if 'num_posts' in key:
            result[0] = d[key]
        if 'num_' in key and 'post' not in key:
            result[1] = d[key]
        if 'per' in key:
            result[2] = d[key]
        if 'unique' in key:
            result[3] = d[key]
    return result


@pytest.mark.parametrize('summary_overview, test_overview',
                         zip(sorted(summary_overviews.items()),
                             sorted(test_overviews.items())),
                         ids=test_ids)
def test_has_correct_overview(summary_overview, test_overview):
    assert dict2overview_list(summary_overview[1]) == test_overview[1]


def test_extract_puts_str_in_list():
    result = extract('#one #two #three', regex=r'#\w+', key_name='hashtag')
    assert result['hashtags'] == ['#one #two #three'.split()]


def test_extract_words_puts_str_in_list():
    word_summary_str = extract_words(word_posts, 'rain',  True)
    assert word_summary_str['top_words'][0][0] == 'rain'


def test_extract_numbers_works_without_separators():
    result = extract_numbers('123,456 hello ', number_separators=None)
    assert result['numbers'] == [['123', '456']]


def test_extract_numbers_handles_dash_in_the_middle_of_seps():
    result = extract_numbers('123,456-789', number_separators=('.', '-', ','))
    assert result['numbers'] == [['123,456-789']]



from advertools.ad_from_string import ad_from_string


def test_len_result_one_more_than_len_slots():
    for i in range(5):
        result = ad_from_string('sample text', slots=[10 for i in range(i)])
        assert len(result) == i + 1


def test_result_same_as_input_text():
    input_text = 'This is sample text to input'
    result = ad_from_string(input_text)
    assert ' '.join(result).strip() == input_text


def test_long_string_gets_skipped():
    result = ad_from_string('This_is_looooooooooooooooooooong')
    assert result[0] == ''


def test_capitalizes_or_not():
    s = 'This text will be split by the Function'
    capitalized = ad_from_string(s, capitalize=True)
    assert capitalized == ['This Text Will Be Split By The',
                           'Function', '', '', '', '', '', '']
    not_capitalized = ad_from_string(s, capitalize=False)
    assert not_capitalized == ['This text will be split by the',
                               'Function', '', '', '', '', '', '']


def test_result_lengths_within_slots():
    s = 'some random text that will be split by different slot lengths'
    for i in range(10):
        slots = [i, i*3, i*5, i*10]
        result = ad_from_string(s, slots=slots)
        for string, slot in zip(result, slots):
            assert len(string) <= slot



"""
Download, Parse, and Analyze XML Sitemaps
=========================================

One of the fastest and easiest ways to get insights on a website's content is
to simply download its XML sitemap(s).

It basically contains a log of their publishing activity, and if they have rich
URLs then you can do some good analysis on their content across time as well.

The :func:`sitemap_to_df` is very simple to use, and only requires the URL of a
sitemap, or a sitemap index. It goes through the sitemap(s) and returns a
DataFrame containing the tags and their information.

Let's go through a quick example of what can be done with sitemaps. We can
start by getting one of the BBC's sitemaps.

>>> bbc_sitemap = sitemap_to_df('https://www.bbc.com/sitemaps/https-sitemap-com-archive-1.xml')
>>> bbc_sitemap
                                                     loc                    lastmod                                            sitemap
0      https://www.bbc.com/arabic/middleeast/2009/06/...  2009-06-20 14:10:48+00:00  https://www.bbc.com/sitemaps/https-sitemap-com...
1      https://www.bbc.com/arabic/middleeast/2009/06/...  2009-06-20 21:07:43+00:00  https://www.bbc.com/sitemaps/https-sitemap-com...
2      https://www.bbc.com/arabic/business/2009/06/09...  2009-06-22 12:41:48+00:00  https://www.bbc.com/sitemaps/https-sitemap-com...
3      https://www.bbc.com/arabic/multimedia/2009/06/...  2009-06-24 15:27:24+00:00  https://www.bbc.com/sitemaps/https-sitemap-com...
4      https://www.bbc.com/arabic/business/2009/06/09...  2009-06-18 15:32:54+00:00  https://www.bbc.com/sitemaps/https-sitemap-com...
                                                  ...                        ...                                                ...
49994  https://www.bbc.com/vietnamese/world/2009/09/0...  2009-09-02 11:46:23+00:00  https://www.bbc.com/sitemaps/https-sitemap-com...
49995  https://www.bbc.com/vietnamese/world/2009/09/0...  2009-09-04 11:20:42+00:00  https://www.bbc.com/sitemaps/https-sitemap-com...
49996  https://www.bbc.com/vietnamese/world/2009/09/0...  2009-09-02 02:40:41+00:00  https://www.bbc.com/sitemaps/https-sitemap-com...
49997  https://www.bbc.com/vietnamese/football/2009/0...  2009-09-02 03:09:06+00:00  https://www.bbc.com/sitemaps/https-sitemap-com...
49998  https://www.bbc.com/vietnamese/world/2009/09/0...  2009-09-05 04:38:11+00:00  https://www.bbc.com/sitemaps/https-sitemap-com...
[49999 rows x 3 columns]

>>> bbc_sitemap.dtypes
loc                     object
lastmod    datetime64[ns, UTC]
sitemap                 object
dtype: object

Since ``lastmod`` is a ``datetime`` object, we can easily use it for various
time-related operations.
Here we look at how many articles have been published (last modified) per year.

>>> bbc_sitemap.set_index('lastmod').resample('A')['loc'].count()
lastmod
2008-12-31 00:00:00+00:00     2261
2009-12-31 00:00:00+00:00    47223
2010-12-31 00:00:00+00:00        0
2011-12-31 00:00:00+00:00        0
2012-12-31 00:00:00+00:00        0
2013-12-31 00:00:00+00:00        0
2014-12-31 00:00:00+00:00        0
2015-12-31 00:00:00+00:00        0
2016-12-31 00:00:00+00:00        0
2017-12-31 00:00:00+00:00        0
2018-12-31 00:00:00+00:00        0
2019-12-31 00:00:00+00:00      483
2020-12-31 00:00:00+00:00       32
Freq: A-DEC, Name: loc, dtype: int64

As the majority are in 2009 with a few in other years, it seems these were
later updated, but we would have to check to verify (in this special case BBC's
URLs contain date information, which can be compared to ``lastmod`` to check if
there is a difference between them).

We can take a look at a sample of the URLs to get the URL template that they
use.

>>> bbc_sitemap['loc'].sample(10).tolist()
['https://www.bbc.com/russian/rolling_news/2009/06/090628_rn_pakistani_soldiries_ambush',
 'https://www.bbc.com/urdu/pakistan/2009/04/090421_mqm_speaks_rza',
 'https://www.bbc.com/arabic/middleeast/2009/07/090723_ae_silwan_tc2',
 'https://www.bbc.com/portuguese/noticias/2009/07/090729_iraquerefenbritsfn',
 'https://www.bbc.com/portuguese/noticias/2009/06/090623_egitomilitaresfn',
 'https://www.bbc.com/portuguese/noticias/2009/03/090302_gazaconferenciaml',
 'https://www.bbc.com/portuguese/noticias/2009/07/090715_hillary_iran_cq',
 'https://www.bbc.com/vietnamese/culture/2009/04/090409_machienhuu_revisiting',
 'https://www.bbc.com/portuguese/noticias/2009/05/090524_paquistaoupdateg',
 'https://www.bbc.com/arabic/worldnews/2009/06/090629_om_pakistan_report_tc2']

It seems the pattern is

    **https://www.bbc.com/{language}/{topic}/{YYYY}/{MM}/{YYMMDD_article_title}**

This is quite a rich structure, full of useful information. We can easily count
how many articles they have by language, by splitting by "/" and getting the
elements at index three, and counting them.

>>> bbc_sitemap['loc'].str.split('/').str[3].value_counts()
russian       14022
persian       10968
portuguese     5403
urdu           5068
mundo          5065
vietnamese     3561
arabic         2984
hindi          1677
turkce          706
ukchina         545
Name: loc, dtype: int64

We can also get a subset of articles written in a certain language, and see how
many articles they publish per month, week, year, etc.

>>> (bbc_sitemap[bbc_sitemap['loc']
...  .str.contains('/russian/')]
...  .set_index('lastmod')
...  .resample('M')['loc'].count())
lastmod
2009-04-30 00:00:00+00:00    1506
2009-05-31 00:00:00+00:00    2910
2009-06-30 00:00:00+00:00    3021
2009-07-31 00:00:00+00:00    3250
2009-08-31 00:00:00+00:00    2769
                             ...
2019-09-30 00:00:00+00:00       8
2019-10-31 00:00:00+00:00      17
2019-11-30 00:00:00+00:00      11
2019-12-31 00:00:00+00:00      24
2020-01-31 00:00:00+00:00       6
Freq: M, Name: loc, Length: 130, dtype: int64

The fifth element after splitting URLs is the topic or category of the article.
We can do the same and count the values.

>>> bbc_sitemap['loc'].str.split('/').str[4].value_counts()[:30]
rolling_news          9044
world                 5050
noticias              4224
iran                  3682
pakistan              2103
afghanistan           1959
multimedia            1657
internacional         1555
sport                 1350
international         1293
india                 1285
america_latina        1274
business              1204
cultura_sociedad       913
middleeast             874
worldnews              872
russia                 841
radio                  769
science                755
football               674
arts                   664
ciencia_tecnologia     627
entertainment          621
simp                   545
vietnam                539
economia               484
haberler               424
interactivity          411
help                   354
ciencia                308
Name: loc, dtype: int64

Finally, we can take the last element after splitting, which contains the slugs
of the articles, replace underscores with spaces, split, concatenate all, put
in a ``pd.Series`` and count the values. This way we see how many times each
word occurred in an article.

>>> (pd.Series(
...     bbc_sitemap['loc']
...     .str.split('/')
...     .str[-1]
...     .str.replace('_', ' ')
...     .str.cat(sep=' ')
...     .split()
...    )
...     .value_counts()[:15])
rn        8808
tc2       3153
iran      1534
video      973
obama      882
us         862
china      815
ir88       727
russia     683
si         640
np         638
afghan     632
ka         565
an         556
iraq       554
dtype: int64
"""
from gzip import GzipFile
import logging
from xml.etree import ElementTree
from urllib.request import urlopen, Request

import pandas as pd

logging.basicConfig(level=logging.INFO)


def sitemap_to_df(sitemap_url):
    """
    Retrieve all URLs and other available tags of a sitemap and put them in a
    DataFrame.

    You can also pass the URL of a sitemap index file.

    :param url sitemap_url: The URL of a sitemap, either a regular sitemap or a
                            sitemap index. In the case of a sitemap index, the
                            function will go through all the sub sitemaps and
                            retrieve all the included URLs in one DataFrame.
    :return sitemap_df: A pandas DataFrame containing all URLs, as well as
                        other tags if available (``lastmod``, ``changefreq``,
                        ``priority``, ``alternate``).
    """
    if sitemap_url.endswith('xml.gz'):
        xml_text = urlopen(Request(sitemap_url,
                                   headers={'Accept-Encoding': 'gzip'}))
        xml_text = GzipFile(fileobj=xml_text)
    else:
        xml_text = urlopen(sitemap_url)
    tree = ElementTree.parse(xml_text)
    root = tree.getroot()

    sitemap_df = pd.DataFrame()

    if root.tag.split('}')[-1] == 'sitemapindex':
        for elem in root:
            for el in elem:
                if el.text.split('.')[-1] in ['xml', 'gz']:
                    try:
                        logging.info(msg='Getting ' + el.text)
                        sitemap_df = sitemap_df.append(sitemap_to_df(el.text),
                                                       ignore_index=True)
                    except Exception as e:
                        logging.warning(msg=str(e) + el.text)
                        error_df = pd.DataFrame(dict(sitemap=el.text),
                                                index=range(1))
                        sitemap_df = sitemap_df.append(error_df,
                                                       ignore_index=True)

    else:
        logging.info(msg='Getting ' + sitemap_url)
        for elem in root:
            d = {}
            for el in elem:
                tag = el.tag
                name = tag.split('}', 1)[1] if '}' in tag else tag

                if name == 'link':
                    if 'href' in el.attrib:
                        d.setdefault('alternate', []).append(el.get('href'))
                else:
                    d[name] = el.text.strip() if el.text else ''
            if 'alternate' in d:
                d['alternate'] = ', '.join(d['alternate'])
            elem_df = pd.DataFrame(d, index=range(1))
            sitemap_df = sitemap_df.append(elem_df, ignore_index=True)
    sitemap_df['sitemap'] = [sitemap_url] if sitemap_df.empty else sitemap_url
    if 'lastmod' in sitemap_df:
        sitemap_df['lastmod'] = pd.to_datetime(sitemap_df['lastmod'], utc=True)
    if 'priority' in sitemap_df:
        sitemap_df['priority'] = sitemap_df['priority'].astype(float)
    return sitemap_df
